@@ -16,6 +16,7 @@
 #                 False for fail and None for not applicable (meaning it will
 #                 not be counted).
 
+import sys, os
 from docutils.core import publish_parts
 from docutils.utils import SystemMessage
 
@@ -31,52 +32,6 @@ LEVELS = ["I don't think that's really cheese",
           "Cottage Cheese",
           "Your cheese is so fresh most people think it's a cream: Mascarpone"]
 
-# All modules in stdlib. No attempt to differentiate between different
-# versions is made at the moment, but this test is made here rather than in
-# the projectdata.py so that it is possible to extend the test to differentiate
-# on Python version if so desired.
-# __main__ is not strictly a part of stdlib, but it's importable...
-STDLIB = set([
-'_abcoll', 'abc', 'aifc', 'antigravity', 'anydbm', 'argparse', 'ast', 
-'asynchat', 'asyncore', 'atexit', 'audiodev', 'base64', 'BaseHTTPServer', 
-'Bastion', 'bdb', 'binhex', 'bisect', 'calendar', 'CGIHTTPServer', 'cgi', 
-'cgitb', 'chunk', 'cmd', 'codecs', 'codeop', 'code', 'collections', 'colorsys',
-'commands', 'compileall', 'ConfigParser', 'contextlib', 'cookielib', 'Cookie',
-'copy', 'copy_reg', 'cProfile', 'csv', 'dbhash', 'decimal', 'difflib',
-'dircache', 'dis', 'doctest', 'DocXMLRPCServer', 'dumbdbm', 'dummy_threading',
-'dummy_thread', 'filecmp', 'fileinput', 'fnmatch', 'formatter', 'fpformat',
-'fractions', 'ftplib', 'functools', '__future__', 'genericpath', 'getopt',
-'getpass', 'gettext', 'glob', 'gzip', 'hashlib', 'heapq', 'hmac',
-'htmlentitydefs', 'htmllib', 'HTMLParser', 'httplib', 'ihooks', 'imaplib',
-'imghdr', 'imputil', 'inspect', 'io', 'keyword', 'linecache', 'locale',
-'_LWPCookieJar', 'macpath', 'macurl2path', 'mailbox', 'mailcap', 'markupbase',
-'md5', 'mhlib', 'mimetools', 'mimetypes', 'MimeWriter', 'mimify',
-'modulefinder', '_MozillaCookieJar', 'multifile', 'mutex', 'netrc', 'new',
-'nntplib', 'ntpath', 'nturl2path', 'numbers', 'opcode', 'optparse',
-'os2emxpath', 'os', 'pdb', 'pickle', 'pickletools', 'pipes', 'pkgutil', 
-'platform', 'plistlib', 'popen2', 'poplib', 'posixfile', 'posixpath', 'pprint',
-'profile', 'pstats', 'pty', 'pyclbr', 'py_compile', 'pydoc', '_pyio', 'Queue',
-'quopri', 'random', 'repr', 're', 'rexec', 'rfc822', 'rlcompleter',
-'robotparser', 'runpy', 'sched', 'sets', 'sgmllib', 'sha', 'shelve', 'shlex',
-'shutil', 'SimpleHTTPServer', 'SimpleXMLRPCServer', 'site', 'smtpd', 'smtplib',
-'sndhdr', 'socket', 'SocketServer', 'sre_compile', 'sre_constants', 'sre_parse',
-'sre', 'ssl', 'stat', 'statvfs', 'StringIO', 'stringold', 'stringprep', 
-'string', '_strptime', 'struct', 'subprocess', 'sunaudio', 'sunau', 'symbol', 
-'symtable', 'sys', 'sysconfig', 'tabnanny', 'tarfile', 'telnetlib', 'tempfile',
-'textwrap', 'this', '_threading_local', 'threading', 'timeit', 'toaiff', 
-'tokenize', 'token', 'traceback', 'trace', 'tty', 'types', 'unittest', 
-'urllib2', 'urllib', 'urlparse', 'UserDict', 'UserList', 'user', 'UserString', 
-'uuid', 'uu', 'warnings', 'wave', 'weakref', '_weakrefset', 'webbrowser', 
-'whichdb', 'xdrlib', 'xmllib', 'xmlrpclib', 'zipfile', 'dl', 'zipimport', 
-'logging', 'lib2to3', 'distutils', 'multiprocessing', 'imp', 'email',
-'bdist_egg', 'time', 'operator', 'array', 'marshal', 'cStringIO', 'datetime',
-'__main__'])
-
-# Some packages will install other modules. This is bad form, but it happens.
-# If you install setuptools you also get a 'pkg_resources' module, for example.
-ALIASES = {'setuptools': ['pkg_resources'],
-           }
-
 class BaseTest(object):
     fatal = False
     
@@ -87,7 +42,7 @@ class FieldTest(BaseTest):
         return bool(data.get(self.field))
     
     def message(self):
-        return ("Your package does not have %s data" % self.field) + (self.fatal and '!' or '')
+        return ("Your package does not have %s data" % self.field) + (self.fatal and '!' or '.')
 
 class Name(FieldTest):
     fatal = True
@@ -111,7 +66,7 @@ class Description(BaseTest):
         if self.fatal:
             return 'The package had no description!'
         else:
-            return 'The packages description should be longer than 10 characters'
+            return 'The packages description should be longer than 10 characters.'
 
 class LongDescription(BaseTest):    
     weight = 50
@@ -120,7 +75,7 @@ class LongDescription(BaseTest):
         return len(data.get('long_description')) > 100
     
     def message(self):
-        return 'The packages long_description is quite short'
+        return 'The packages long_description is quite short.'
 
 class Classifiers(FieldTest):
     weight = 100
@@ -169,8 +124,8 @@ class PythonVersion(BaseTest):
     def message(self):
         if self._major_version_specified:
             return "The classifiers should specify what minor versions of "\
-                   "Python you support as well as what major version"
-        return "You should specify what Python versions you support"
+                   "Python you support as well as what major version."
+        return "You should specify what Python versions you support."
         
         
 class Keywords(FieldTest):    
@@ -212,43 +167,35 @@ class TestSuite(FieldTest):
         return "Setuptools and Distribute support running tests. By "\
                "specifying a test suite, it's easy to find and run tests "\
                "both for automated tools and humans."
-
-class Dependencies(BaseTest):
+    
+class RunnableTests(BaseTest):
     
     def test(self, data):
-        if data.get('_source_download') is False:
-            # Could not download source from PyPI. Skip this.
+        if not '_testresult' in data:
             return None
-        declared_dependencies = data.get('install_requires', []) + \
-                                data.get('tests_require', []) + \
-                                data.get('setup_requires', [])
-        for r in data.get('extras_require', {}).values():
-            declared_dependencies.extend(r)
-
-        if not declared_dependencies: 
-            # No dependencies declared. If it *has* dependencies, this is 
-            # bad form and gives a lot of minus.
-            self.weight = 200
-        else:
-            # It has declared dependencies, perhaps it forgot to add new ones,
-            # perhaps some dependencies are optional. We give it less weight
-            # because they may be optional:
-            self.weight = 50
-
-        # Add the packages in the module to the declared dependencies.
-        declared_dependencies.extend(data['packages'])       
-
-        # Update the declared with aliases:
-        for d in declared_dependencies:
-            declared_dependencies.extend(ALIASES.get(d, []))
-            
-        dependencies = data['_imports'] - STDLIB
-        self._undeclared = dependencies - set(declared_dependencies)
-        return not bool(self._undeclared)
         
+        if data['_testresult']== 'Failure':
+            self.weight = 100
+            self._msg = "The test suite failed!"
+            return False
+        if data['_testresult'] == 'WrongPython':
+            self.weight = 0
+            self._msg = "This project doesn't support this version of Python; tests not run."
+            return False
+        if data['_testresult'] == 'NoTests':
+            self.weight = 0
+            self._msg = "This package is not set up to run tests."
+            return False
+        if data['_testresult'] == 'Success':
+            self.weight = 100
+            self._msg = ""
+            return True
+        self.fatal = True
+        self._msg = "Uknown error, this is likely a Pyroma bug."
+        return False
+    
     def message(self):
-        undeclared = ', '.join(self._undeclared)
-        return "Did you forget to declare the following dependencies?: " + undeclared
+        return self._msg
 
 class PackageDocs(BaseTest):
     weight = 0 # Just a recommendation
@@ -260,7 +207,7 @@ class PackageDocs(BaseTest):
         return "The site packages.python.org is a nice place to put your "\
                "documentation that makes it easy to find, and relieves you of "\
                "hosting it. You should consider using it."
-
+    
 class ValidREST(BaseTest):
     
     weight = 50
@@ -270,7 +217,6 @@ class ValidREST(BaseTest):
         try:
             parts = publish_parts(source=source, writer_name='html4css1')
         except SystemMessage, e:
-            import pdb;pdb.set_trace()
             self._message = e.args[0].strip()
             return False
         
@@ -279,6 +225,29 @@ class ValidREST(BaseTest):
     def message(self):
         return 'Your long_description is not valid ReST: ' + self._message
     
+class BusFactor(BaseTest):
+
+    
+    def test(self, data):
+        if not '_owners' in data:
+            self.weight = 0
+            return None
+
+        if len(data['_owners']) == 1:
+            self.weight = 100
+            return False
+    
+        if len(data['_owners']) == 2:
+            self.weight = 50
+            return False
+        
+        # Three or more, that's good.
+        self.weight = 100
+        return True
+    
+    def message(self):
+        return "You should have three or more owners of the project on PyPI."
+    
 ALL_TESTS = [
     Name(),
     Version(),
@@ -293,9 +262,10 @@ ALL_TESTS = [
     License(),
     ZipSafe(),
     TestSuite(),
-    #Dependencies(),
+    RunnableTests(),
     PackageDocs(),
     ValidREST(),
+    BusFactor(),
 ]
 
 def rate(data):
@@ -309,10 +279,11 @@ def rate(data):
     for test in ALL_TESTS:
         res = test.test(data)
         if res is False:
-            bad += test.weight
-            fails.append(test.message())
             if test.fatal:
                 fatality = True
+            else:
+                bad += test.weight
+                fails.append(test.message())
         elif res is True:
             if not test.fatal:
                 good += test.weight
